@@ -52,6 +52,7 @@ def _update_params(hyperopt_params, params):
                     hyperopt_params[k] = v
     return hyperopt_params
 
+
 def opt_catboost(X_train, y_train, cat_features=None, cv_folds=3, n_trials=20, verbose=0, params=None, device='GPU', max_rounds=5000, seed=0, early_stopping=20):
     """
     Performs n_trials using Hyperopt to obtain the best catboost parameters
@@ -76,7 +77,7 @@ def opt_catboost(X_train, y_train, cat_features=None, cv_folds=3, n_trials=20, v
                        'bagging_temperature': hp.uniform('bagging_temperature', 0, 1),
                        'random_strength': hp.uniform('random_strength', 0.5, 3),
                        'depth': hp.quniform('depth', 2, 8, 2),  # 10,
-                       'rsm': hp.uniform('rsm', 0.1, 0.9), #colsample_bylevel
+                       'rsm': hp.uniform('rsm', 0.1, 0.9) if device != 'GPU' else None, #colsample_bylevel
                        'loss_function': 'RMSE',
                        'eval_metric': 'RMSE',
                        'max_bin': hp.qloguniform('border_count', np.log(32), np.log(64), 5), #border count
@@ -121,7 +122,7 @@ def opt_catboost(X_train, y_train, cat_features=None, cv_folds=3, n_trials=20, v
 
         if verbose > 0:
             print('Train Loss: %0.4f, Test Loss: %0.4f RMSE with %d iterations. Time elapsed %s' % (
-            train_loss_value, test_loss_value, best_iteration, str(time.time() - start)))
+            train_loss_value, test_loss_value, best_iteration, str(round(time.time() - start,2))))
 
         val = {'loss': test_loss_value,  # mandatory
                'status': STATUS_OK,  # mandatory
@@ -137,7 +138,7 @@ def opt_catboost(X_train, y_train, cat_features=None, cv_folds=3, n_trials=20, v
                        max_evals=n_trials,
                        trials=trials)
 
-    best_params, best_n = _get_best_params(hyperopt_cb, HYPEROPT_PARAMS, INTEGER_PARAMS_LIST)
+    best_params, best_n = _get_best_params(hyperopt_cb, HYPEROPT_PARAMS, INTEGER_PARAMS_LIST, trials)
 
     # Plot results
     if verbose == 2:
@@ -215,7 +216,7 @@ def opt_xgboost(X_train, y_train, cat_features=None, cv_folds=3, n_trials=20, ve
 
         if verbose > 0:
             print('Train Loss: %0.4f, Test Loss: %0.4f RMSE with %d iterations. Time elapsed %s' % (
-            train_loss_value, test_loss_value, best_iteration, str(time.time() - start)))
+            train_loss_value, test_loss_value, best_iteration, str(round(time.time() - start,2))))
 
         val = {'loss': test_loss_value,  # mandatory
                 'status': STATUS_OK,  # mandatory
@@ -231,7 +232,7 @@ def opt_xgboost(X_train, y_train, cat_features=None, cv_folds=3, n_trials=20, ve
                        max_evals=n_trials,
                        trials=trials)
 
-    best_params, best_n = _get_best_params(hyperopt_xgb, HYPEROPT_PARAMS, INTEGER_PARAMS_LIST)
+    best_params, best_n = _get_best_params(hyperopt_xgb, HYPEROPT_PARAMS, INTEGER_PARAMS_LIST, trials)
 
     # Plot results
     if verbose == 2:
@@ -239,4 +240,95 @@ def opt_xgboost(X_train, y_train, cat_features=None, cv_folds=3, n_trials=20, ve
 
     return {'params': best_params, 'n': best_n}
 
-#def opt_lightgbm(X_train, y_train, cat_features=None, cv_folds=3, n_trials=20, verbose=0, params=None, device='GPU', max_rounds=5000, seed=0, early_stopping=20):
\ No newline at end of file
+
+def opt_lightgbm(X_train, y_train, cat_features=None, cv_folds=3, n_trials=20, verbose=0, params=None, device='CPU', max_rounds=5000, seed=0, early_stopping=20):
+    """
+    Performs n_trials using Hyperopt to obtain the best LightGBM parameters
+
+    Inputs:
+        X_train, y_train, cat_features: Data to build Pool object
+        cv_folds: number of CV folds for validation on each trial
+        n_trials: number of trials to perform
+        verbose: 0 = no log and no plot, 1 = log but no plot, 2 = log and plot
+        params: dict to override the Hyperopt params
+        device: 'gpu' of 'cpu' (lowercase)
+        max_rounds: max number of iterations to train every trial
+        seed: random state seed
+        early_stopping: early stopping rounds
+
+    Outputs:
+        dict that contains the best params and the best number of iterations
+    """
+    HYPEROPT_PARAMS = {'objective': 'tweedie',
+                                'tweedie_variance_power': hp.uniform('tweedie_variance_power', 1.0, 2.0),
+                                'learning_rate': hp.loguniform('learning_rate', np.log(0.01), np.log(0.3)),
+                                'num_leaves': hp.choice('num_leaves', [2 ** i - 1 for i in range(3, 13)]),
+                                'max_depth': hp.quniform('max_depth', 2, 10, 2),  # 10,
+                                'min_data_in_leaf': hp.choice('min_data_in_leaf', [2 ** i - 1 for i in range(3, 13)]),
+                                # 'min_child_weight': hp.loguniform('min_child_weight', -16, 5)
+                                'bagging_fraction': hp.uniform('bagging_fraction', 0.2, 0.9),
+                                'bagging_freq': hp.uniform('bagging_freq', 1, 10),
+                                'feature_fraction': hp.uniform('feature_fraction', 0.2, 0.9),
+                                'lambda_l1': hp.loguniform('lambda_l1', 0, np.log(100)),
+                                'lambda_l2': hp.loguniform('lambda_l2', 0, np.log(20)),
+                                'max_bin': hp.choice('max_bin', [2 ** i - 1 for i in range(3, 11)]),
+                                'metric': 'rmse',
+                                'seed': hp.choice('seed', [0, 1, 2, 3]),
+                                'num_threads': 5,
+                                # 'gpu_id': 0,
+                                'device_type ':device.lower()
+                                }
+
+    HYPEROPT_PARAMS = _update_params(HYPEROPT_PARAMS, params)
+    INTEGER_PARAMS_LIST = ['max_depth', 'seed']
+
+    dataset = lgb.Dataset(X_train, label=y_train, categorical_feature =cat_features)
+
+    def objective(params):
+        start = time.time()
+
+        # Control Integer Params don't go float
+        for par_name in INTEGER_PARAMS_LIST:
+            params[par_name] = int(np.round(params[par_name]))
+
+        cv = lgb.cv(params=params,
+                    train_set=dataset,
+                    nfold=cv_folds,
+                    stratified=False,
+                    metrics=params['eval_metric'],
+                    verbose_eval=False,
+                    seed=seed,
+                    num_boost_round=max_rounds,
+                    early_stopping_rounds=early_stopping)
+
+        # Metric to extract the loss from
+        cv_results = pd.DataFrame(cv)
+        metric_mean_name = 'rmse-mean'
+        best_iteration = cv_results[metric_mean_name].idxmin()
+        loss = cv_results[metric_mean_name].iloc[best_iteration]
+
+        if verbose > 0:
+            print('Test Loss: %0.4f RMSE with %d iterations. Time elapsed %s' % (
+            loss, best_iteration, str(round(time.time() - start,2))))
+
+        val = {'loss': loss,  # mandatory
+               'status': STATUS_OK,  # mandatory
+               'best_n_iters': best_iteration}
+
+        return val
+
+    trials = Trials()
+    hyperopt_lgb = fmin(fn=objective,
+                       space=HYPEROPT_PARAMS,
+                       algo=tpe.suggest,
+                       verbose=True if verbose > 0 else False,
+                       max_evals=n_trials,
+                       trials=trials)
+
+    best_params, best_n = _get_best_params(hyperopt_lgb, HYPEROPT_PARAMS, INTEGER_PARAMS_LIST, trials)
+
+    # Plot results
+    if verbose == 2:
+        _evaluation_plot(trials.losses())
+
+    return {'params': best_params, 'n': best_n}
\ No newline at end of file
